library ieee;

use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;

entity alu2 is
	port (clk,nw: in std_logic;
	din: in std_logic_vector(31 downto 0);
	op: in std_logic_vector( 2 downto 0);
	R: out std_logic_vector(63 downto 0);
	ready : out std_logic );
end alu2;

architecture beh of alu2 is
type stati is (idle, decode, exe_h, exe_w, smult,read1,read2);
signal S: stati:=idle;
signal read1_en,read2_en,exeh_en,exew_en,smult_en: std_logic;
signal ope: std_logic_vector(2 downto 0);
signal d0,d1,d2: std_logic_vector(31 downto 0);
signal conta,conta_max: integer range 0 to 4;
begin
	process(clk)
	begin
		if clk'event and clk='0' then
			case S is
				when idle => if nw = '1' then
					            S<= decode;
				             else S<= idle;
							 end if;
			   when decode =>  case ope is
				                when "000" |"011" | "110" | "111" =>  S<= read1;
				   				when "001" | "010" | "100"   =>  S <= exe_h;
								when others => S<= idle;
			                  end case;
			 when read1 => if ope = "111" then
				               S<= read2;
			                else S<= exe_w;
				 		   end if;
			when read2 => S<= smult;
			when exe_h =>   if conta < conta_max then
			                 	S<= exe_h;
			                else S<= idle;
							end if;
			when exe_w =>   if conta < conta_max then
			                 	S<= exe_w;
			                else S<= idle;
							end if;	
		   when smult =>   if conta < conta_max then
			                 	S<= smult;
			                else S<= idle;
							end if;
			when others => S<= idle;
		   end case;
		end if;
	end process;
	
  process (S)
  
  begin
	  case S is
		  when idle => 
		  read1_en <= '0';
		  read2_en <= '0';
		  exeh_en<='0';
		  exew_en<= '0';
		  smult_en<='0';
		 when decode => 
		   read1_en <= '0';
		  read2_en <= '0';
		  exeh_en<='0';
		  exew_en<= '0';
		  smult_en<='0';	   
		  case ope is
			  when "000"|"001" => conta_max<= 1;
			  when "100" |"110" => conta_max <= 3;
			  when "111" => conta_max<= 4;
			  when others => conta_max<= 0;	
		end case;
		when read1 =>
		   read1_en <= '1';
		  read2_en <= '0';
		  exeh_en<='0';
		  exew_en<= '0';
		  smult_en<='0';  
	  when read2 => 
	     read1_en <= '0';
		  read2_en <= '1';
		  exeh_en<='0';
		  exew_en<= '0';
		  smult_en<='0';
     when exe_h =>
	    read1_en <= '0';
		  read2_en <= '0';
		  exeh_en<='1';
		  exew_en<= '0';
		  smult_en<='0'; 
	when exe_w => 
	     read1_en <= '0';
		  read2_en <= '0';
		  exeh_en<='0';
		  exew_en<= '1';
		  smult_en<='0';  
   when smult => 
        read1_en <= '0';
		  read2_en <= '0';
		  exeh_en<='0';
		  exew_en<= '0';
		  smult_en<='1'; 
 end case;
 end process;
 
 process(clk)
 begin
	 if clk'event and clk='0' then
		if nw='1' then 
			 d0 <= din;
			 ope <= op;
			 conta<= 0;
		end if;	   
		
		if read1_en='1' then 
			d1 <= din;
		end if;
		
		if read2_en='1' then
			d2 <= din;
		end if;	   
		
		
		  	
		  if exeh_en ='1' then
			if conta=conta_max then
			  if ope = "001" then
				R<=   conv_std_logic_vector(conv_integer(D0(31 downto 16))+conv_integer(d0(15 downto 0)),64);
			  elsif ope = "010" then
				  R(15 downto 0) <= D0(31 downto 16) or D0(15 downto 0);
				  R(63 downto 16) <= (others => '0');
			  else R<=   conv_std_logic_vector(conv_integer(D0(31 downto 16))*conv_integer(d0(15 downto 0)),64);
			  end if;
			    conta <= 0;
			else conta <= conta+1;
			end if;
		 end if;
		  
		 if exew_en ='1' then
		   if conta=conta_max then
			  if ope = "000" then
				R<=   conv_std_logic_vector(conv_integer(D0)+conv_integer(D1),64);
			  elsif ope = "011" then
				  R(31 downto 0) <= D0 or D1;
				  R(63 downto 32) <= (others => '0');
			  else R<=   conv_std_logic_vector(conv_integer(D0)*conv_integer(d1),64);
			  end if;
			  
			  conta <= 0;
		   else conta <= conta+1;
				 
		   end if;
			  
		 end if;
		 
		 if smult_en = '1' then
		    if conta=conta_max then 
			    R <=  conv_std_logic_vector(conv_integer(D0)+conv_integer(d1)*conv_integer(d2),64);
 		        
				conta <= 0; 
	        else   conta <= conta+1;
		        
		    end if;
	     end if;	
		 
		 if  conta=conta_max and (exeh_en ='1' or  exew_en ='1' or smult_en = '1' ) then
			 ready <= '1';
		 else ready <='0';
		 end if;	 
		 
	end if;
 end process;
 end beh;
 